
/*
SIETS CGI-BIN API Client Library
For connecting from C/C++ applications to SIETS API
For MS Windows based applications only.
(c) 2004 Lursoft IT
*/

#ifndef SCBAPI_H
#define SCBAPI_H

#include <windows.h>

#define SCBAPI_ERR_UPLIM -100 // upper error code limit
#define SCBAPI_ERR_INVURL -101 // invalid url syntax
#define SCBAPI_ERR_CONNECT -102 // connect error
#define SCBAPI_ERR_REQUEST -103 // request error
#define SCBAPI_ERR_REQSEND -104 // error sending request
#define SCBAPI_ERR_HTTP -105 // http protocol error
#define SCBAPI_ERR_LWLIM -110 // lower error code limit

/*
Turns on debug mode. In debug mode last request sent to SIETS
and last response received will be dumped to text file.
*/
#ifdef _DEBUG
#define SCBAPI_DEBUG // debug mode
#endif // _DEBUG

/* 
Size, in bytes, receive buffer is increased during response
reding from SIETS server.
*/
#define SCBAPI_INCREASE 10240

/*
Macro to check whether char* returned from connection functions
contains error code.
*/
#define SCBAPI_IS_ERROR(R) (((int)(R)>SCBAPI_ERR_UPLIM || (int)(R)<SCBAPI_ERR_LWLIM) ? 0 : 1)

typedef struct
{
	unsigned int code;
	char *text;
	char *level;
	char *source;
}
SCBAPI_SIETS_ERROR;

/*
Initializes SCBAPI library.
Must bee called once per application, before any other function is called.
*/
int SCBAPI_Init();

/*
Frees SCBAPI library.
Must be called once per application, during termination.
*/
int SCBAPI_Free();

/*
Send tsData data to SIETS CGI-BIN API, located in tsURL.
*/
char *SCBAPI_SendRecv(char *tsURL, char *tsData);

/* Get last HTTP error. Error message as string can be read
if tsBuffer is not NULL and iBufSize is large enough.
Returns response received.
*/
int SCBAPI_GetError(char *tsBuffer, int iBufSize);

/*
Generic SIETS command.
Returns SIETS response.
*/
char *SCBAPI_Command(char *tsURL, char *tsCommand, char *tsContent, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Status command.
*/
char *SCBAPI_Status(char *tsURL, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Insert command.
tsOptional contains additional data, that must be contained within
document, but cannot be located in text tag.
*/
char *SCBAPI_Insert(char *tsURL, char *tsID, char *tsTitle, unsigned long iRate, char *tsText, char *tsInfo, char *tsSpectags, char *tsOptional, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Update command.
tsOptional contains additional data, that must be contained within
document, but cannot be located in text tag.
*/
char *SCBAPI_Update(char *tsURL, char *tsID, char *tsTitle, unsigned long iRate, char *tsText, char *tsInfo, char *tsSpectags, char *tsOptional, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Clear command
*/
char *SCBAPI_Clear(char *tsURL, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Retrieve command
*/
char *SCBAPI_Retrieve(char *tsURL, char *tsID, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Lookup command
*/
char *SCBAPI_Lookup(char *tsURL, char *tsID, char *tsEncoding, char *tsStorage, char *tsUser, char *tsPassword);

/*
Extracts error from SIETS response. Returns 0, if there if no error
or 1 - otherwise. sError can be NULL, if error details not required.
*/
int SCBAPI_SietsError(char *tsResponse, SCBAPI_SIETS_ERROR *sError);

/*
Frees sError structure.
*/
void SCBAPI_SietsErrorFree(SCBAPI_SIETS_ERROR *sError);

/*
Helper function, extracts XML tag from tsData.
nr - reserved for future use, pass 0.
*/
char *SCBAPI_ExtractTag(char *tsData, char *tsTag, unsigned int nr);

/*
Encode text to be valid XML.
*/
char *SCBAPI_XMLEntitiesEncode(char *tsString);

/*
Replace some symbols in text to be valid in spectags.
*/
void SCBAPI_ReplaceSpecial(char **tsData);

/*
Extract and display error from tsResponse.
hWnd - handle to message box parent window.
Returns 1 - if there were error, 0 - otherwise.
*/
int SCBAPI_DisplayError(HWND hWnd, char *tsResponse);

#endif // SCBAPI_H
